var debug = require('debug')('checkout-index');
var Remotes = require('remotes');
var netrc = require('netrc');
var mkdirp = require('mkdirp');
var thunkify = require('thunkify');
var join = require('path').join;
var folder = process.env.REMOTE_FOLDER || process.env.HOME + '/remotes';
var Downloader = require('component-downloader');
var consoler = require('component-consoler');
var fs = require('graceful-fs');
var semver = require('semver');
var rimraf = require('rimraf');
rimraf = thunkify(rimraf);

var mkdir = thunkify(mkdirp);

function getAuth() {
  var github = netrc()['github.com'];
  if (github) return github.login + ':' + github.password;
  var name = process.env.GITHUB_USERNAME;
  var pass = process.env.GITHUB_PASSWORD;
  return name + ':' + pass;
}

var remote = new Remotes.github({
  timeout: 10000,
  retries: 2,
  auth: getAuth()
});

var downloader = Downloader({
  dir: folder,
  remote: remote,
  concurrency: 10, // 10 files at a time
  verbose: true,
});

function* checkout(name, tag, clean) {
  var args = name.split('/');
  var versions = yield getVersions(name, tag);
  if (clean !== false && ~versions.indexOf('master')) {
    yield rimraf(join(folder, args[0], args[1], 'master'));
  }
  debug('downloading tags: ' + versions);
  for (var i = 0, len = versions.length; i < len; i++) {
    var ref = versions[i];
    try {
      yield* downloader(name, ref);
    } catch (e){
      //probably not exist any more
      consoler.log('download failed ', name + '@' + ref);
    }
  }
  return versions;
}

function* getVersions(name, tag) {
  var versions;
  var args = name.split('/');
  var dir = join(folder, args[0], args[1]);
  if (!tag) {
    versions = yield* remote.versions(name);
    versions = yield* filterTags(dir, versions);
    versions.unshift('master');
  } else if (tag != 'master' && exists(dir, tag)){
    versions = [];
  } else {
    versions = [tag];
  }
  return versions;
}

//only valid and newer tags
function* filterTags(dir, tags) {
  if (!fs.existsSync(dir)) return tags;
  var dirs = fs.readdirSync(dir);
  dirs = dirs.filter(onlySemver);
  dirs.sort(sortBySemver);
  var latest = dirs[0];
  debug('Latest tag version: ' + latest);
  tags = tags.filter(function(tag) {
    if (latest && semver.gte(latest, tag)) return false;
    return semver.valid(tag);
  })
  //only last five
  return tags.slice(0, 5);
}

function onlySemver(v) {
  return semver.valid(v);
}

function sortBySemver(a, b) {
  return semver.lt(a, b) ? 1 : -1;
}

function exists(dir, tag) {
  return fs.existsSync(join(dir, tag)) ||
         fs.existsSync(join(dir, 'v' + tag));
}

module.exports = checkout;
